import re
import os
import networkx
from networkx.algorithms.components.connected import connected_components

f=open('H1N1-16273-one-line.fasta','r')

arr=[]
dseq={}
while True:
	line=f.readline().strip()
	if not line:break
	if re.search('>',line):
		name=line
		dseq[name]=''
	else:
		dseq[name]=line

for s in dseq:
	arr.append([s,dseq[s]])

def to_graph(l):
	G=networkx.Graph()
	for part in l:
		G.add_nodes_from(part)
		G.add_edges_from(to_edges(part))
	return G

def to_edges(l):
	it=iter(l)
	last=next(it)

	for current in it:
		yield last,current
		last=current

G=to_graph(arr)
o=open('H1N1-16273-one-line-removeSame.fasta','w+')
o2=open('H1N1-16273-one-line-removeSame.clstr','w+')
for ele in list(connected_components(G)):
	ele=list(ele)
	tem=[]
	for e in ele:
		if re.search('>',e):
			tem.append(e)
	o.write(tem[0]+'\n'+dseq[tem[0]]+'\n')
	tem2=[]
	for t in tem:
		l=len(dseq[t])
		s=t+':'+str(l)
		tem2.append(s)
	outs='\t'.join(tem2)
	o2.write(outs+'\n')



import re
import os
import getopt
import sys
from Bio import AlignIO
import collections
import math
import matplotlib.pyplot as plt
import networkx
from networkx.algorithms.components.connected import connected_components
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC


input_msa=''
column_file=''
cls_file=''
dash_cutoff=0.01
k=25
opts,args=getopt.getopt(sys.argv[1:],"i:c:r:")

for opt,arg in opts:
	if opt=='-i':
		input_msa=arg
	if opt=='-c':
		column_file=arg
	if opt=='-r':
		cls_file=arg	

def estimate_shannon_entropy(dna_seq):
	#m=len(dna_seq)
	m=0
	base_dict={'a':'','t':'','g':'','c':'','A':'','T':'','G':'','C':''}
	bases_raw=dict(collections.Counter([tmp_base for tmp_base in dna_seq]))
	''' we will not consider dash or special characters when we calculate entropy '''
	bases={}
	for b in bases_raw:
		if b not in base_dict:continue
		bases[b]=bases_raw[b]
		m+=bases_raw[b]
	shannon_entropy_value=0
	for base in bases:
		n_i=bases[base]
		p_i=n_i / float(m)
		entropy_i=p_i * (math.log(p_i))
		shannon_entropy_value+=entropy_i
	return shannon_entropy_value * (-1), bases_raw

def estimate_shannon_entropy_spe(alignment,strain_set,left_c):
	m=0
	base_dict={'a':'','t':'','g':'','c':'','A':'','T':'','G':'','C':''}
	centropy={}
	cbase_freq={}
	for c in left_c:
		seq=''
		i=0
		for n in alignment[:,c]:
			if i not in strain_set:
				i+=1
				continue
			seq+=n
			i+=1
		entropy,base_count=estimate_shannon_entropy(seq)
		cbase_freq[c]=base_count
		if entropy==0:
			centropy[c]=1000
		else:
			centropy[c]=entropy
	#print(centropy,cbase_freq)
	return centropy,cbase_freq
	
def estimate_shannon_entropy_iterate(alignment,used_strains,used_columns,centropy):
	new_centropy={}
	new_cbase_freq={}
	base_dict={'a':'','t':'','g':'','c':'','A':'','T':'','G':'','C':''}
	for c in centropy:
		tem_bf={}
		m=0
		if c in used_columns:
			new_centropy[c]=1000
			continue
		else:
			i=0
			for s in alignment[:,c]:
				if i in used_strains:
					i+=1
					continue
				if s not in base_dict:
					i+=1
					continue
				if s not in tem_bf:
					tem_bf[s]=1
					m+=1
					i+=1
				else:
					tem_bf[s]+=1
					m+=1
					i+=1
		shannon_entropy_value=0
		for base in tem_bf:
			n_i=tem_bf[base]
			p_i=n_i / float (m)
			entropy_i=p_i * (math.log(p_i))
			shannon_entropy_value+=entropy_i
		new_centropy[c]=shannon_entropy_value * (-1)
		new_cbase_freq[c]=tem_bf
	return new_centropy, new_cbase_freq
 
		

#def cls_with_entropy_hier(alignment, centropy ,cbase_freq, sid, sname2seq, sid2name, sname2id):
def cls_with_entropy_hier(alignment,left_c,strain_set):
	used_strains={}
	used_columns={}
	selected_columns={}
	#copyentropy=centropy # copyentropy will be updated during the iterative process
	#copyalignment=alignment # copyalignment will not be updated
	base_dict={'a':'','t':'','g':'','c':'','A':'','T':'','G':'','C':''}
	#o1=open('output_test.txt','w+')
	#o1.write('finish inputting '+str(len(sname2id))+' sequences from the alignment\n')
	cqnum=len(left_c)
	snum=len(strain_set)
	sarr=sorted(list(strain_set.keys()))
	centropy, cbase_freq=estimate_shannon_entropy_spe(alignment,strain_set,left_c)
	#exit()
	ibreak=0
	while True:
		if len(used_strains)==snum or len(used_columns)==cqnum:break
		print('Progress: C: ',len(used_columns),'/',cqnum,' S:',len(used_strains),'/',snum)
		res=sorted(centropy.items(),key=lambda d:d[1])
		if res[0][1]==1000 and ibreak==0:
			return {},{},{},[]		
		ibreak+=1
		if res[0][1]==1000:break
		## Select columns with minimum entropy
		for r in res:	
			if r[0] not in used_columns: # If the column is already used, we will not consider it again
				current_c=r[0]	# This is the column id, the column with minimum entropy
				break
		rawseq=''
		sid2ni={}
		i=0
		for s in sarr:		
			rawseq+=alignment[s,current_c]
			sid2ni[i]=s
			i+=1
		#base_freq_raw=dict(collections.Counter([tmp_base for tmp_base in seq]))
		#base_freq={}
		'''
		for b in base_freq_raw:
			if b in base_dict:
				base_freq[b]=base_freq_raw[b]
		'''
		res2=sorted(cbase_freq[current_c].items(),key=lambda d:d[1])
		round_strain=[]	
		for r in res2:
			if r[0] not in base_dict:continue
			if r[0]==res2[-1][0]:continue
			strain_id=[sid2ni[i] for i, letter in enumerate(rawseq) if letter == r[0]]
			round_strain=round_strain+strain_id
			# Output cls info
			selected_columns[current_c]=''
			'''
			o1.write('column '+str(current_c))
			for b in ['a','t','g','c']:
				if b not in cbase_freq[current_c]:
					o1.write(' '+b+'(0)')
				else:
					o1.write(' '+b+'('+str(cbase_freq[current_c][b])+')')
			o1.write('\n')
			for s in strain_id:
				o1.write('>'+sid2name[s]+' ')
			o1.write(str(current_c)+' '+r[0]+'\n')
			'''
			cbase_freq[current_c][r[0]]=0
		for s in round_strain:
			used_strains[s]=''
		# record used columns 
		used_columns[current_c]=''
		# update entropy and cbase_freq
		centropy, cbase_freq=estimate_shannon_entropy_iterate(alignment,used_strains,used_columns,centropy)
	## Cluster based on selected column
	sub_info={} # Sub-cluster-ID -> Strain_Name
	kmr_sub={} # kmer -> sub-cluster
	kmr_pos={} # kmer -> pos-snp
	strain_sub={}
	sort_sub=[]
	if len(selected_columns)==0:
		return 	sub_info,kmr_sub,kmr_pos,sort_sub
	else:
		# For cluster
		def to_graph(l):
			G=networkx.Graph()
			for part in l:
				G.add_nodes_from(part)
				G.add_edges_from(to_edges(part))
			return G
		def to_edges(l):
			it=iter(l)
			last=next(it)
			for current in it:
				yield last,current
				last=current
	
		carr=sorted(list(selected_columns.keys()))
		scseq=[]
		for s in strain_set:
			tem=[]
			cseq=''
			for c in carr:
				cseq+=alignment[s,c]
			tem.append(cseq)
			tem_s=str(s)+'_cls'
			tem.append(tem_s)
			scseq.append(tem)
		#print(scseq)
		#exit()
		G=to_graph(scseq)
		count=1
		for ele in list(connected_components(G)):
			ele=list(ele)
			pre='SubCls'+str(count)+'_'+str(len(ele)-1)
			sort_sub.append(pre)
			sub_info[pre]=[]
			for e in ele:
				if not re.search('_',e):continue
				e=re.sub('_cls','',e)
				e=int(e)
				sub_info[pre].append(e)
				strain_sub[e]=pre
			count+=1
		# Ready to extract kmer for each sub-cluster
		check_dict={'a':'A','t':'T','g':'G','c':'C','A':'A','T':'T','G':'G','C':'C','-':''}
		dmap={'a':'A','t':'T','g':'G','c':'C','A':'A','T':'T','G':'G','C':'C'}
		for c in carr:
			for s in strain_set:
				base=alignment[s,c].upper()
				left=alignment[s].seq[:c][::-1]
				right=alignment[s].seq[c+1:]
				pos_base=str(c)+'-'+base
				if c-12<0:
					block_seq=alignment[s].seq[0:c]+base+alignment[s].seq[c+1:c+13]
				else:
					'''
					print(c+13,c-12,base)
					print(alignment[s].seq[c-12:c])
					print(alignment[s].seq[c+1:c+13])
					print(len(alignment[s,:]))
					exit()
					'''
					block_seq=alignment[s].seq[c-12:c]+base+alignment[s].seq[c+1:c+13]
				check_num=0
				for b in block_seq:
					if b not in check_dict:
						check_num=1
						break
				if check_num==1:continue
				lseq=''
				rseq=''
				for l in left:
					if len(lseq)==(k-1)/2:
						break
					if l in dmap:
						lseq+=dmap[l]
					elif not l=='-':
						lseq+=l.upper()
				for r in right:
					if len(rseq)==(k-1)/2:
						break
					if r in dmap:
						rseq+=dmap[r]
					elif not r=='-':
						rseq+=r.upper()
				lseq=lseq[::-1]
				kmr=lseq+base+rseq
				if len(kmr) <25:
					if len(lseq)<(k-1)/2:
						rseq=''
						rl=k-1-len(lseq)
						for r in right:
							if len(rseq)==rl:
								break
							if r in dmap:
								rseq+=dmap[r]
							elif not r=='-':
								rseq+=r.upper()
					if len(rseq)<(k-1)/2:
						lseq=''
						ll=k-1-len(rseq)
						for l in left:
							if len(lseq)==ll:
								break
							if l in dmap:
								lseq+=dmap[l]
							elif not l=='-':
								lseq+=l.upper()
					kmr=lseq+base+rseq
				if kmr not in kmr_sub:
					kmr_sub[kmr]={strain_sub[s]:''}
				else:
					kmr_sub[kmr][strain_sub[s]]=''
				if kmr not in kmr_pos:
					kmr_pos[kmr]={pos_base:''}
				else:
					kmr_pos[kmr][pos_base]=''
		return  sub_info,kmr_sub,kmr_pos,sort_sub				
# Load used column info
cfile=open(column_file,'r')
bused_c={}
while True:
	line=cfile.readline().strip()
	if not line:break
	if not re.search('column',line):continue
	ele=line.split()
	bused_c[int(ele[1])]=''
# Load msa
alignment=AlignIO.read(input_msa,'fasta')
#entropy,base_count=estimate_shannon_entropy(alignment[:,7])
sid=range(len(alignment[:,0]))
sname=[]
sname2seq={}  # Strain_Name -> Strain_msa_sequence
cnum=0
for record in alignment:
	name=record.id.strip()
	sname.append('>'+name)	
	cnum=len(record.seq)
	sname2seq[name]=record.seq
cid=range(cnum)
centropy={} # Column_Entropy
cid_seq={} # Column_ID -> Column_Sequence
cbase_freq={}
# Here we can filter some columns
for column in cid:
	seq=alignment[:,column]
	entropy,base_count=estimate_shannon_entropy(seq)
	if '-' not in base_count:
		if not entropy==0:
			centropy[column]=entropy 
			#cid_seq[column]=seq
			cbase_freq[column]=base_count
	#res=sorted(raw_base.items(),key=lambda d:d[1],reverse=True)
	else:
		if base_count['-']<= dash_cutoff * len(sid):
			#centropy.append(entropy)
			if not entropy==0:
				#cid_seq[column]=seq
				cbase_freq[column]=base_count
				centropy[column]=entropy
sid2name=dict(zip(sid,sname))  # Strain_ID -> Strain_Name
sname2id=dict(zip(sname,sid))  # Strain_Name -> Strain_ID
## Check how many columns left
stat=0
left_c={}
for c in centropy:
	if c not in bused_c:
		left_c[c]=''
		stat+=1
print('Total columns: ',cnum)
print('Used column in first iterate: ',len(bused_c))
print('Left Column: ',stat,'/',len(centropy))
## We need to parse the cluster, and exclude used strains
#finished_strains={} # ID -> Strain_Name
strain_info={} # Strain_Name -> Cluster_info
cls_strain={} # Cluster -> Strain_Name -> Strain_ID
cls_rep={}
strain_rep={} # Strain_Name -> Rep_Strain
fc=open(cls_file,'r')
while True:
	line=fc.readline().strip()
	if not line:break
	if re.search('Cluster',line):
		cls=re.sub('>','',line)
		cls_strain[cls]={}
	else:
		ele=line.split('\t')
		if re.search('\*',line):
			rep=ele[0]
			#cls_rep[cls]=rep
			strain_rep[rep]=rep
			strain_info[rep]=cls
			cls_strain[cls][rep]=int(ele[-1])
			cls_rep[cls]=rep
		else:
			strain_rep[ele[0]]=rep
			strain_info[ele[0]]=cls
			cls_strain[cls][ele[0]]=int(ele[-1])

#print(cls_strain['Cluster14_2'],strain_rep['>gb:CY067031'],strain_info['>gb:CY067031'])

#exit()
### Most important part ----> Split cluster
o1=open('Strain_cls_info.txt','w+')
o2=open('Rebuild_cls.clstr','w+')
o3=open('SubCls_kmer.txt','w+')
raw_cls_size=[]
new_cls_size=[]
for c in cls_strain:
	raw_cls_size.append(len(cls_strain[c]))
	if len(cls_strain[c])==1:
		new_cls_size.append(len(cls_strain[c]))
		for s in cls_strain[c]:
			o1.write(s+'\t'+strain_rep[s]+'\t'+strain_info[s]+'\t'+strain_info[s]+'\t'+str(sname2id[s])+'\t1\n')
			o2.write('>'+c+'\n')
			o2.write('\t'+s+'\t*\t'+str(sname2id[s])+'\n')
	else:
		strain_set={}
		for s in  cls_strain[c]:
			strain_set[cls_strain[c][s]]=''
		sub_info,kmr_sub,kmr_pos,sort_sub=cls_with_entropy_hier(alignment,left_c,strain_set)
		print(sub_info, sort_sub)
		#exit()
		if len(sub_info)==0 or len(sort_sub)==1:
			new_cls_size.append(len(cls_strain[c]))
			o2.write('>'+c+'\n')
			for s in cls_strain[c]:
				o1.write(s+'\t'+strain_rep[s]+'\t'+strain_info[s]+'\t'+strain_info[s]+'\t'+str(sname2id[s])+'\t'+str(len(cls_strain[c]))+'\n')
				if s==cls_rep[c]:
					o2.write('\t'+s+'\t*\t'+str(sname2id[s])+'\n')
				else:
					o2.write('\t'+s+'\t'+str(sname2id[s])+'\n')
		else:
			o2.write('>'+c+'\n')
			for sub in sort_sub:
				o2.write('\t>'+sub+'\n')
				new_cls_size.append(len(sub_info[sub]))
				for ss in sub_info[sub]:
					ss=sid2name[ss]
					o1.write(ss+'\t'+strain_rep[ss]+'\t'+strain_info[ss]+'\t'+sub+'\t'+str(sname2id[ss])+'\t'+str(len(sub_info[sub]))+'\n')
					if ss==cls_rep[c]:
						o2.write('\t\t'+ss+'\t*\t'+str(sname2id[ss])+'\n')
					else:
						o2.write('\t\t'+ss+'\t'+str(sname2id[ss])+'\n')
			for kl in kmr_sub:
				info1=','.join(list(kmr_pos[kl].keys()))
				info2=','.join(list(kmr_sub[kl].keys()))
				o3.write(kl+'\t'+c+'\t'+info1+'\t'+info2+'\n')
				seq=Seq(kl,IUPAC.ambiguous_dna)
				k2=seq.reverse_complement()._data
				o3.write(k2+'\t'+c+'\t'+info1+'\t'+info2+'\n')
				

plt.hist(raw_cls_size,bins=100)
plt.xlabel('Cluster size')
plt.ylabel('Cluster Number')
plt.savefig('Before_split.png')	
plt.figure()
plt.hist(new_cls_size,bins=100)
plt.xlabel('Cluster size')
plt.ylabel('Cluster Number')
plt.savefig('After_split.png')
### Now we need to iterate all columns
#cls_with_entropy(alignment, centropy, cbase_freq, sid, sname2seq, sid2name, sname2id)



import re
import os,sys
import argparse
from distutils.sysconfig import get_python_lib

__author__="Liao Herui"

usage="VirStrain - An RNA virus strain-level identification tool for short reads."

def main():
	parser=argparse.ArgumentParser(prog='VirStrain.py',description=usage)
	parser.add_argument('-i','--input_reads',dest='input_reads',type=str,required=True,help="Input fastq data --- Required")
	parser.add_argument('-p','--input_reads2',dest='input_reads2',type=str,help="Input fastq data for PE reads.")
	parser.add_argument('-d','--database_dir',dest='db_dir',type=str,required=True,help='Database dir --- Required')
	parser.add_argument('-o','--output_dir',dest='out_dir',type=str,help='Output dir (default: current dir/VirStrain_Out)')
	parser.add_argument('-c','--site_filter_cutoff',dest='sf_cutoff',type=str,help='The cutoff of filtering one site (default: 0.05)')
	parser.add_argument('-m','--high_mutation_virus',dest='hm_virus',help='If the virus has high mutation rate, use this option. (default: Not use)',default='1',nargs='?')
	
	args=parser.parse_args()	
	in_read1=args.input_reads
	in_read2=args.input_reads2
	db_dir=args.db_dir
	out_dir=args.out_dir
	sfc=args.sf_cutoff
	hm=args.hm_virus
	if not sfc:
		sfc=0.05
	if not out_dir:
		out_dir='VirStrain_Out'
	if not os.path.exists(out_dir):
		os.makedirs(out_dir)
	spk=get_python_lib()+'/VirStrain/'
	# whether this is the virus with high mutation rate, like HIV
	if not hm:
		if not in_read2:
			os.system('python '+spk+'S3_Strain_pred_My_Method_For_HIV.py -i '+in_read1+' -s '+db_dir+'/Pos-snp-kmer-all.txt  -f '+db_dir+'/Pos-snp-kmer-all.fa -c '+db_dir+'/Remove_redundant_matrix_MM_Call.clstr -d '+db_dir+'/ID2Name.txt  -b '+db_dir+' -o VirStrain_report.txt')
		else:
			os.system('python '+spk+'S3_Strain_pred_My_Method_For_HIV.py -i '+in_read1+' -p '+in_read2+' -s '+db_dir+'/Pos-snp-kmer-all.txt  -f '+db_dir+'/Pos-snp-kmer-all.fa -c '+db_dir+'/Remove_redundant_matrix_MM_Call.clstr -d '+db_dir+'/ID2Name.txt  -b '+db_dir+' -o VirStrain_report.txt')
		if os.path.exists('VirStrain_report.txt'):
			os.system('python '+spk+'S4_Plot_strain_cov.py')
			os.system('mv VirStrain_report.txt VirStrain_report.html Mps_ps_depth.csv Ops_ps_depth.csv '+out_dir)
			exit()
	# Start to identify strains
	if not in_read2: # SE reads
		os.system('python '+spk+'S3_Strain_pred_My_Method_V0819_Val.py -i '+in_read1+' -s '+db_dir+'/Pos-snp-kmer-all.txt -m '+db_dir+'/Strain_pos_snp_matrix_not_redundant_MM_Call.txt -f '+db_dir+'/Pos-snp-kmer-all.fa -c '+db_dir+'/Strain_cls_info.txt -b '+db_dir+'/SubCls_kmer.txt -d '+db_dir+' -o VirStrain_report.txt')
	else: # PE reads
		os.system('python '+spk+'S3_Strain_pred_My_Method_V0819_Val.py -i '+in_read1+' -p '+in_read2+' -s '+db_dir+'/Pos-snp-kmer-all.txt -m '+db_dir+'/Strain_pos_snp_matrix_not_redundant_MM_Call.txt -f '+db_dir+'/Pos-snp-kmer-all.fa -c '+db_dir+'/Strain_cls_info.txt -b '+db_dir+'/SubCls_kmer.txt -d '+db_dir+' -o VirStrain_report.txt')
	# Plot Html Page
	
	if os.path.exists('VirStrain_report.txt'):
		os.system('python '+spk+'S4_Plot_strain_cov.py')	
	
	os.system('mv VirStrain_report.txt VirStrain_report.html  Mps_ps_depth.csv Ops_ps_depth.csv '+out_dir)



if __name__=='__main__':
	sys.exit(main())



import re
import os
import pandas as pd
import numpy as np
#import matplotlib.pyplot as plt
from _plotly_future_ import v4_subplots
import plotly.plotly as py
import  plotly.graph_objs as go
from plotly.subplots import make_subplots
#from plotly import tools

sfl=pd.read_csv('Mps_ps_depth_HIV.csv')
sf2=pd.read_csv('Ops_ps_depth_HIV.csv')
#print(sfl.columns)
# Get 
s=[]
for e in list(sfl.columns):
	if re.search('ID',e):continue
	name=re.split('_',e)[0]
	if name not in s:
		s.append(name)
so=[]
for e in list(sf2.columns):
	if re.search('ID',e):continue
	if re.search('None',e):continue
	name=re.split('_',e)[0]
	if name not in so:
		so.append(name)

#fig=plt.figure(figsize=(17,6))
#ax=fig.add_subplot(111)
#width=0.2

#sfl.plot('ID',s[0]+'_Freq',kind='bar')
#plt.xlabel("")
#plt.savefig('Test_cov.png')
#fig=py.iplot({'data':[Bar(x=sfl['ID'],y=sfl[s[0]+'_Freq'])],'layout':{'margin':{'b':300},'xaxis':{'tickangle':50}}})
#fig = tools.make_subplots(rows=2, cols=1, subplot_titles=['Depth', 'Label_Number'], shared_xaxes=True, shared_yaxes=False)
#traces=[go.Bar(x=sfl['ID'],y=sfl[s[0]+'_Freq']),go.Bar(x=sfl['ID'],y=sfl[s[0]+'_LNum'])]
# Extract 0 freq
i=0
zx=[]
zy=[]
for sw in sfl[s[0]+'_Freq']:
	if sw==0: 
		zx.append(sfl['ID'][i])
		zy.append(sw)
	i+=1
ft=open('Test.html','w+')
fig=go.Figure()
fig = make_subplots(specs=[[{"secondary_y": True}]])
fig.add_trace(go.Bar(x=sfl['ID'],y=sfl[s[0]+'_Freq'],name='depth'),secondary_y=False,)
#fig.add_trace(go.Scatter(x=zx,y=zy,mode='markers',opacity=0.5,marker=dict(size=5),name='Zero depth pos'),secondary_y=False,)
fig.add_trace(go.Scatter(x=sfl['ID'],y=(-1)*sfl[s[0]+'_LNum'],mode='lines+markers',opacity=0.5,marker=dict(size=5),name='(-1)*(strain number)'),secondary_y=True,)
fig.add_trace(go.Scatter(x=zx,y=zy,mode='markers',opacity=0.5,marker=dict(size=5,color='red'),name='Zero depth pos'),secondary_y=False,)
#fig2=go.Figure()
#fig2.add_trace(go.Bar(x=sfl['ID'],y=(sfl[s[0]+'_LNum'])))
'''
fig.append_trace(traces[0],1,1)
fig.append_trace(traces[1],2,1)
fig.layout.update(autosize=False,width=1000,height=500,)
mi=sfl[s[0]+'_LNum'].min()
ma=sfl[s[0]+'_LNum'].max()
print(mi)
fig['layout']['yaxis2'].update(title='',range=[mi,ma],autorange=False)
'''
ma=sfl[s[0]+'_LNum'].max()
ma2=sfl[s[0]+'_Freq'].max()
fig.layout.update(autosize=False,width=1200,height=300,title={'text':'Most Possible Strain: '+s[0],'xanchor': 'center'})
#fig['layout']['yaxis2']['autorange']
#fig['layout']['yaxis2']['autorange'] = "reversed"
fig.update_yaxes(range=[(-3)*ma,3*ma],secondary_y=True,)
#fig['layout']['yaxis2']['autorange'] = "reversed"
fig.update_yaxes(range=[(-1)*ma2,ma2],secondary_y=False,)
#with open('Test.html','a') as ft:
ft.write(fig.to_html(full_html=False, include_plotlyjs='cdn'))
#ft.write(fig.to_html(full_html=False, include_plotlyjs='cdn'))
#ft.write(fig.to_html(full_html=False, include_plotlyjs='cdn'))
#ft.write(fig.to_html(full_html=False, include_plotlyjs='cdn'))
#fig.write_html('Test.html')




import re
import os
import getopt
import sys
import numpy as np
import json

opts,args=getopt.getopt(sys.argv[1:],"i:p:s:f:c:o:d:b:")
read_1=''
read_2=''
input_cls_file=''
out_file=''
id_map_file=''
input_kmr_file=''
db_dir=''
depth_cutoff=0.01

for opt,arg in opts:
	if opt=='-i':
		read_1=arg
	elif opt=='-p':
		read_2=arg
	elif opt=='-c':
		input_cls_file=arg
	elif opt=='-o':
		out_file=arg
	elif opt=='-d':
		id_map_file=arg
	elif opt=='-f':
		input_kmr_file=arg
	elif opt=='-b':
		db_dir=arg
dskc={} # Strain -> kmer number countdskc={} 
'''
Two dict used to visualize the depth figure
'''
dscl={} # Strain -> Column ->Label
dscf={} # Strain -> Column -> Freq
id2name={}
fi=open(id_map_file,'r')
while True:
	line=fi.readline().strip()
	if not line:break
	ele=line.split('\t')
	id2name[ele[0]]=ele[1]
	dscl[ele[0]]={}
	dscf[ele[0]]={}
	dskc[ele[0]]=0
id2cls={}
fc=open(input_cls_file,'r')
while True:
	line=fc.readline().strip()
	if not line:break
	if re.search('Cluster',line):
		cls=re.sub('>','',line)
	else:
		i=line.split('\t')[-1]
		id2cls[i]=cls
		
file_dir=sys.path[0]

#f=open(input_kmr_txt,'r')
dks={} #  kmer -> Strain  -> Column -> 0
dss={} # Strain -> map score
#dskc={} # Strain -> kmer number count
'''
Two dict used to visualize the depth figure
'''
#dscl={} # Strain -> Column ->Label
#dscf={} # Strain -> Column -> Freq
'''
while True:
	line=f.readline().strip()
	if not line:break
	ele=line.split('\t')
	dks[ele[0]]={}
	strain=re.split(',',ele[-1])
	column_raw=re.split(',',ele[1])
	column=[]
	for c in column_raw:
		rc=int(re.split('-',c)[0])
		column.append(rc)
	for s in strain:
		dks[ele[0]][s]={}
		dss[s]=0
		
		if s not in dscl:
			dscl[s]={}
		if s not in dscf:
			dscf[s]={}
		if s not in dskc:
			dskc[s]=0
	
		dskc[s]+=1
		#continue
		for c in column:
			#rc=int(re.split('-',c)[0])
			dscl[s][c]=len(strain)
			dscf[s][c]=0
			dks[ele[0]][s][c]=0
'''
dscl=json.load(open(db_dir+'/dscl.json'))
dscf=json.load(open(db_dir+'/dscf.json'))
dks=json.load(open(db_dir+'/dks.json'))
dss=json.load(open(db_dir+'/dss.json'))
dskc=json.load(open(db_dir+'/dskc.json'))
'''
json.dump(dscl,open("dscl.json",'w'))
json.dump(dscf,open("dscf.json",'w'))
json.dump(dks,open('dks.json','w'))
json.dump(dss,open('dss.json','w'))
json.dump(dskc,open('dskc.json','w'))
'''
print('Load Dict -> Done')
#exit()

if read_2=='':
	cmd1=file_dir+'/jellyfish-linux count -m 25 -s 100M -t 8 --if '+input_kmr_file+' -o Tem_VS.jf '+read_1
	cmd2=file_dir+'/jellyfish-linux dump -c Tem_VS.jf > Tem_Vs.fa'
	os.system(cmd1)
	os.system(cmd2)
else:
	cmd1=file_dir+'/jellyfish-linux count -m 25 -s 100M -t 8 --if '+input_kmr_file+' -o Tem_VS.jf '+read_1+' '+read_2
	cmd2=file_dir+'/jellyfish-linux dump -c Tem_VS.jf > Tem_Vs.fa'
	os.system(cmd1)
	os.system(cmd2)

fp=open('Tem_Vs.fa','r')
total=[]
while True:
	line=fp.readline().strip()
	if not line:break
	ele=line.split()
	if int(ele[-1])==0:continue
	total.append(int(ele[-1]))
min_depth=depth_cutoff*np.mean(total)

f2=open('Tem_Vs.fa','r')
dkc={}
dsvkc={} # Strain -> Valid kmer number count
#total=0
#ot=open('All_kmer_count.txt','w+')
while True:
	line=f2.readline().strip()
	if not line:break
	ele=line.split()
	if int(ele[-1])==0:continue
	#ss=','.join(dks[ele[0]])
	if int(ele[-1])<min_depth:continue
	#ot.write(line+'\t'+ss+'\n')
	dkc[ele[0]]=int(ele[-1])
	#total+=int(ele[-1])
	for s in dks[ele[0]]:
		if s not in dsvkc:
			dsvkc[s]={}
		#dsvkc[s]+=1
		
		#dss[s]+=float(ele[-1])/(float(len(dks[ele[0]])))
		dss[s]+=int(ele[-1])
		for c in dks[ele[0]][s]:
			dsvkc[s][c]=''
			dscf[s][c]+=int(ele[-1])
#print(len(dkc))
#exit()
dc={}
if os.path.exists(db_dir+'/head_file.txt'):
	fp=open(db_dir+'/head_file.txt','r')
	while True:
		line=fp.readline().strip()
		if not line:break
		ele=line.split('\t')
		dc[ele[0]]=ele[1]

o=open(out_file,'w+')
o.write('\t\tStrah_ID\tCls_info\tKmer_Hit_Num\tMap_Rate\tStrain_Depth\tStrain_info\n')
res=sorted(dss.items(),key=lambda d:d[1],reverse=True)
#o.write('Round-1\t'+res[0][0]+'\t'+str(res[0][1])+'\n')
depth_mp=float(res[0][1])/float(dskc[res[0][0]])
o.write('>>Most possible strains:\n')
if id2name[res[0][0]] not in dc:
	o.write('\t\t'+id2name[res[0][0]]+'\t'+id2cls[res[0][0]]+'\t'+str(res[0][1])+'\t'+str(len(dsvkc[res[0][0]]))+'/'+str(len(dscl[res[0][0]]))+'\t'+str(depth_mp)+'\tNA\n')
else:
	o.write('\t\t'+id2name[res[0][0]]+'\t'+id2cls[res[0][0]]+'\t'+str(res[0][1])+'\t'+str(len(dsvkc[res[0][0]]))+'/'+str(len(dscl[res[0][0]]))+'\t'+str(depth_mp)+'\t'+dc[id2name[res[0][0]]]+'\n')
mp_strain=id2name[res[0][0]]
mp_sid=res[0][0]
#exit()
cuid={res[0][0]:''}
'''
o3=open('check_used_kmer.txt','w+')
for k in dkc:
	if  res[0][0] in dks[k]:
		outs=','.join(dks[k])
		o3.write(k+'\t'+str(dkc[k])+'\t'+outs+'\n')
exit()
print(cuid)
'''
o.write('>>Other possible strains(Top5):\n')

c=0
#o2=open('check_rest_kmer.txt','w+')
op_strain=[]
op_sid=[]
while True:
	if c==5:break
	temc={}
	temc_knum={}
	#check=0
	count=0
	for k in dkc:
		check=0
		for cc in cuid:
			if cc in dks[k]:
				check=1
				#total=total-dkc[k]
				break
		if check==1:continue
		count+=1		
		#o2.write(k+'\t'+str(dkc[k])+'\t')
		#outs=','.join(dks[k])
		#o2.write(outs+'\n')
		for s in dks[k]:
			if s not in temc:
				temc[s]=dkc[k]
			else:
				temc[s]+=dkc[k]
			if s not in temc_knum:
				temc_knum[s]=1
				
			else:
				temc_knum[s]+=1
	#print(count)
	#exit()
	#if len(res)==0:break
	res=sorted(temc.items(),key=lambda d:d[1],reverse=True)
	if len(res)==0:break
	#exit()
	#print(res[:50])
	#exit()
	cuid[res[0][0]]=''
	op_sid.append(res[0][0])
	op_strain.append(id2name[res[0][0]])
	d=c+2
	cdepth=float(res[0][1])/float(temc_knum[res[0][0]])
	o.write('\t>>Left_Kmer_Num:'+str(count)+'\n')
	if id2name[res[0][0]] not in dc:
		o.write('\t\t'+str(id2name[res[0][0]])+'\t'+id2cls[res[0][0]]+'\t'+str(res[0][1])+'\t'+str(len(dsvkc[res[0][0]]))+'/'+str(len(dscl[res[0][0]]))+'\t'+str(cdepth)+'\tNA\n')
	else:
		o.write('\t\t'+str(id2name[res[0][0]])+'\t'+id2cls[res[0][0]]+'\t'+str(res[0][1])+'\t'+str(len(dsvkc[res[0][0]]))+'/'+str(len(dscl[res[0][0]]))+'\t'+str(cdepth)+'\t'+dc[id2name[res[0][0]]]+'\n')
	#o.write('Round-'+str(d)+'\t'+str(res[0][0])+'\t'+str(res[0][1])+'\n')
	c+=1
'''
res=sorted(dss.items(),key=lambda d:d[1],reverse=True)
for r in res[:10]:
	if not r[1]==0:
		o.write(r[0]+'\t'+str(r[1])+'\n')
'''
os.system('rm Tem_VS.jf Tem_Vs.fa')
print('Text report is done. Now will generate HTML report!')
ov1=open('Mps_ps_depth.csv','w+')
ov2=open('Ops_ps_depth.csv','w+')
ov1.write('ID,Column_ID,'+mp_strain+'_Freq,'+mp_strain+'_LNum\n')
carr=sorted(list(dscf[mp_sid].keys()))
i=1
for c in carr:
	ov1.write(str(i)+','+str(c)+','+str(dscf[mp_sid][c])+','+str(dscl[mp_sid][c])+'\n')
	i+=1

ov2.write('ID,Column_ID')
if len(op_strain)==0:
	ov2.write(',None,None\n')
else:
	#sid=0
	for s in op_strain:
		#sname=op_strain[sid]
		ov2.write(','+s+'_Freq'+','+s+'_LNum')
	ov2.write('\n')
	i=1
	for c in carr:
		ov2.write(str(i)+','+str(c))
		check=0
		check1=0
		check2=0
		for s in op_sid:
			if c not in dscf[s]:
				check+=1
				check1=1
			if c not in dscl[s]:
				check+=1
				check2=1
			if check==0:
				ov2.write(','+str(dscf[s][c])+','+str(dscl[s][c]))
			elif check==1:
				if check1==1:
					ov2.write(',0,'+str(dscl[s][c]))
				if check2==1:
					ov2.write(','+str(dscf[s][c])+',0')
			else:
				ov2.write(',0,0')
				
		ov2.write('\n')
		i+=1
		#sid+=1



import re
import os
import getopt
import sys
from Bio import AlignIO
import collections
import math


input_msa=''
dash_cutoff=0.01
opts,args=getopt.getopt(sys.argv[1:],"i:")

for opt,arg in opts:
	if opt=='-i':
		input_msa=arg

def estimate_shannon_entropy(dna_seq):
	#m=len(dna_seq)
	m=0
	base_dict={'a':'','t':'','g':'','c':'','A':'','T':'','G':'','C':''}
	bases_raw=dict(collections.Counter([tmp_base for tmp_base in dna_seq]))
	''' we will not consider dash or special characters when we calculate entropy '''
	bases={}
	for b in bases_raw:
		if b not in base_dict:continue
		bases[b]=bases_raw[b]
		m+=bases_raw[b]
	shannon_entropy_value=0
	for base in bases:
		n_i=bases[base]
		p_i=n_i / float(m)
		entropy_i=p_i * (math.log(p_i))
		shannon_entropy_value+=entropy_i
	return shannon_entropy_value * (-1), bases_raw
def estimate_shannon_entropy_iterate(alignment,used_strains,used_columns,centropy):
	new_centropy={}
	new_cbase_freq={}
	base_dict={'a':'','t':'','g':'','c':'','A':'','T':'','G':'','C':''}
	for c in centropy:
		tem_bf={}
		m=0
		if c in used_columns:
			new_centropy[c]=1000
			continue
		else:
			i=0
			for s in alignment[:,c]:
				if i in used_strains:
					i+=1
					continue
				if s not in base_dict:
					i+=1
					continue
				if s not in tem_bf:
					tem_bf[s]=1
					m+=1
					i+=1
				else:
					tem_bf[s]+=1
					m+=1
					i+=1
		shannon_entropy_value=0
		for base in tem_bf:
			n_i=tem_bf[base]
			p_i=n_i / float (m)
			entropy_i=p_i * (math.log(p_i))
			shannon_entropy_value+=entropy_i
		new_centropy[c]=shannon_entropy_value * (-1)
		new_cbase_freq[c]=tem_bf
	return new_centropy, new_cbase_freq
 
		

def cls_with_entropy(alignment, centropy ,cbase_freq, sid, sname2seq, sid2name, sname2id):
	used_strains={}
	used_columns={}
	#copyentropy=centropy # copyentropy will be updated during the iterative process
	#copyalignment=alignment # copyalignment will not be updated
	base_dict={'a':'','t':'','g':'','c':'','A':'','T':'','G':'','C':''}
	o1=open('output_test.txt','w+')
	o1.write('finish inputting '+str(len(sname2id))+' sequences from the alignment\n')
	cqnum=len(centropy)
	snum=len(sname2id)
	while True:
		if len(used_strains)==len(sname2id) or len(used_columns)==cqnum:break
		print('Progress: C: ',len(used_columns),'/',cqnum,' S:',len(used_strains),'/',snum)
		res=sorted(centropy.items(),key=lambda d:d[1])
		## Select columns with minimum entropy
		for r in res:	
			if r[0] not in used_columns: # If the column is already used, we will not consider it again
				current_c=r[0]	# This is the column id, the column with minimum entropy
				break
		rawseq=alignment[:,current_c]
		#base_freq_raw=dict(collections.Counter([tmp_base for tmp_base in seq]))
		#base_freq={}
		'''
		for b in base_freq_raw:
			if b in base_dict:
				base_freq[b]=base_freq_raw[b]
		'''
		res2=sorted(cbase_freq[current_c].items(),key=lambda d:d[1])
		round_strain=[]	
		for r in res2:
			if r[0] not in base_dict:continue
			if r[0]==res2[-1][0]:continue
			strain_id=[i for i, letter in enumerate(rawseq) if letter == r[0]]
			round_strain=round_strain+strain_id
			# Output cls info
			o1.write('column '+str(current_c))
			for b in ['a','t','g','c']:
				if b not in cbase_freq[current_c]:
					o1.write(' '+b+'(0)')
				else:
					o1.write(' '+b+'('+str(cbase_freq[current_c][b])+')')
			o1.write('\n')
			for s in strain_id:
				o1.write('>'+sid2name[s]+' ')
			o1.write(str(current_c)+' '+r[0]+'\n')
			cbase_freq[current_c][r[0]]=0
		for s in round_strain:
			used_strains[s]=''
		# record used columns 
		used_columns[current_c]=''
		# update entropy and cbase_freq
		centropy, cbase_freq=estimate_shannon_entropy_iterate(alignment,used_strains,used_columns,centropy)
		
		
# Load msa
alignment=AlignIO.read(input_msa,'fasta')
#entropy,base_count=estimate_shannon_entropy(alignment[:,7])
sid=range(len(alignment[:,0]))
sname=[]
sname2seq={}  # Strain_Name -> Strain_msa_sequence
cnum=0
for record in alignment:
	name=record.id.strip()
	sname.append(name)	
	cnum=len(record.seq)
	sname2seq[name]=record.seq
cid=range(cnum)
centropy={} # Column_Entropy
cid_seq={} # Column_ID -> Column_Sequence
cbase_freq={}
# Here we can filter some columns
for column in cid:
	seq=alignment[:,column]
	entropy,base_count=estimate_shannon_entropy(seq)
	if '-' not in base_count:
		if not entropy==0:
			centropy[column]=entropy 
			#cid_seq[column]=seq
			cbase_freq[column]=base_count
	#res=sorted(raw_base.items(),key=lambda d:d[1],reverse=True)
	else:
		if base_count['-']<= dash_cutoff * len(sid):
			#centropy.append(entropy)
			if not entropy==0:
				#cid_seq[column]=seq
				cbase_freq[column]=base_count
				centropy[column]=entropy
sid2name=dict(zip(sid,sname))  # Strain_ID -> Strain_Name
sname2id=dict(zip(sname,sid))  # Strain_Name -> Strain_ID
### Now we need to iterate all columns
cls_with_entropy(alignment, centropy, cbase_freq, sid, sname2seq, sid2name, sname2id)



import re
import os
#import json
import sys
import getopt

input_fa=''
cls_file=''
out_dir=''
opts,args=getopt.getopt(sys.argv[1:],"i:c:o:")
for opt,arg in opts:
	if opt=='-i':
		input_fa=arg
	elif opt=='-c':
		cls_file=arg
	elif opt=='-o':
		out_dir=arg
if not out_dir:
	out_dit='VirStrain_DB'
#f=open('head_file.txt','r')
f=open(input_fa)
dhead={}
while True:
	line=f.readline().strip()
	if not line:break
	if not re.search('>',line):continue
	if re.search('\|',line):
		pre=re.split('\|',line)[0].strip()
	else:
		pre=line.split()[0].strip()
	#print(pre)
	#exit()
	info=re.sub('.*human','',line)
	info=re.sub(',.*','',info)
	dhead[pre]=info

#clade=json.loads('nextclade.json')
'''
with open('nextclade.json','r') as j:
	clade=json.loads(j.read())
dclade={}
for c in clade:
	if 'clade' not in c:
		dclade['>'+c['seqName']]='NA'
	else:
		dclade['>'+c['seqName']]=c['clade']
'''

f2=open(cls_file,'r')
o=open(out_dir+'/Anno_rebuild_cls.clstr','w+')
while True:
	line=f2.readline().strip()
	if not line:break
	if re.search('Cluster',line) or re.search('Cls',line):
		if re.search('Cluster',line):
			o.write(line+'\n')
		else:
			o.write('\t'+line+'\n')
	else:
		ele=line.split('\t')
		o.write('\t\t'+line+'\t'+dhead[ele[0]]+'\n')



import re
import os
import getopt
import sys
from Bio import AlignIO
import collections
import math
import matplotlib.pyplot as plt
import networkx
from networkx.algorithms.components.connected import connected_components
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC

src_path=sys.path[0]

input_msa=''

cls_file=''
dash_cutoff=0.0
k=25
opts,args=getopt.getopt(sys.argv[1:],"i:r:")

for opt,arg in opts:
	if opt=='-i':
		input_msa=arg
	if opt=='-r':
		cls_file=arg	

	

#def cls_with_entropy_hier(alignment, centropy ,cbase_freq, sid, sname2seq, sid2name, sname2id):
def cls_with_perl_hier(sstrain2seq,strain_set,src_path):
	selected_columns={}
	#copyentropy=centropy # copyentropy will be updated during the iterative process
	#copyalignment=alignment # copyalignment will not be updated
	cnum=0
	fs=open('Vbuild_tem_cls.aln','w+')
	for s in sstrain2seq:
		fs.write(s+'\n'+sstrain2seq[s]+'\n')
		cnum=len(sstrain2seq[s])
	fs.close()
	os.system('perl '+src_path+'/aln2cluster-overlap-kmer-withd.pl Vbuild_tem_cls.aln '+str(len(strain_set))+' 0 0 '+str(cnum)+' 0 > Vbuild_tem.column')
	#print('perl aln2cluster-overlap-kmer-withd.pl Vbuild_tem_cls.aln '+str(len(strain_set))+' 0 0 '+str(cnum)+' 0 > Vbuild_tem.column')
	#exit()
	fs2=open('Vbuild_tem.column','r')
	while True:
		line=fs2.readline().strip()
		if not line:break
		if not re.search('column',line):continue
		ele=line.split()
		selected_columns[int(ele[1])]=''
	#os.system('rm Vbuild_tem_cls.aln Vbuild_tem.column')
	#exit()
	## Cluster based on selected column
	sub_info={} # Sub-cluster-ID -> Strain_Name
	kmr_sub={} # kmer -> sub-cluster
	kmr_pos={} # kmer -> pos-snp
	strain_sub={}
	sort_sub=[]
	if len(selected_columns)==0:
		return 	sub_info,kmr_sub,kmr_pos,sort_sub
	else:
		# For cluster
		def to_graph(l):
			G=networkx.Graph()
			for part in l:
				G.add_nodes_from(part)
				G.add_edges_from(to_edges(part))
			return G
		def to_edges(l):
			it=iter(l)
			last=next(it)
			for current in it:
				yield last,current
				last=current
	
		carr=sorted(list(selected_columns.keys()))
		scseq=[]
		for s in strain_set:
			tem=[]
			cseq=''
			for c in carr:
				cseq+=sstrain2seq[s][c]
			tem.append(cseq)
			tem_s=str(strain_set[s])+'_cls'
			tem.append(tem_s)
			scseq.append(tem)
		#print(scseq)
		#exit()
		G=to_graph(scseq)
		count=1
		for ele in list(connected_components(G)):
			ele=list(ele)
			pre='SubCls'+str(count)+'_'+str(len(ele)-1)
			sort_sub.append(pre)
			sub_info[pre]=[]
			for e in ele:
				if not re.search('_',e):continue
				e=re.sub('_cls','',e)
				e=int(e)
				sub_info[pre].append(e)
				strain_sub[e]=pre
			count+=1
		# Ready to extract kmer for each sub-cluster
		check_dict={'a':'A','t':'T','g':'G','c':'C','A':'A','T':'T','G':'G','C':'C','-':''}
		dmap={'a':'A','t':'T','g':'G','c':'C','A':'A','T':'T','G':'G','C':'C'}
		for c in carr:
			for s in strain_set:
				base=sstrain2seq[s][c].upper()
				left=sstrain2seq[s][:c][::-1]
				right=sstrain2seq[s][c+1:]
				pos_base=str(c)+'-'+base
				if c-12<0:
					block_seq=sstrain2seq[s][0:c]+base+sstrain2seq[s][c+1:c+13]
				else:
					'''
					print(c+13,c-12,base)
					print(alignment[s].seq[c-12:c])
					print(alignment[s].seq[c+1:c+13])
					print(len(alignment[s,:]))
					exit()
					'''
					block_seq=sstrain2seq[s][c-12:c]+base+sstrain2seq[s][c+1:c+13]
				check_num=0
				for b in block_seq:
					if b not in check_dict:
						check_num=1
						break
				if check_num==1:continue
				lseq=''
				rseq=''
				for l in left:
					if len(lseq)==(k-1)/2:
						break
					if l in dmap:
						lseq+=dmap[l]
					elif not l=='-':
						lseq+=l.upper()
				for r in right:
					if len(rseq)==(k-1)/2:
						break
					if r in dmap:
						rseq+=dmap[r]
					elif not r=='-':
						rseq+=r.upper()
				lseq=lseq[::-1]
				kmr=lseq+base+rseq
				if len(kmr) <25:
					if len(lseq)<(k-1)/2:
						rseq=''
						rl=k-1-len(lseq)
						for r in right:
							if len(rseq)==rl:
								break
							if r in dmap:
								rseq+=dmap[r]
							elif not r=='-':
								rseq+=r.upper()
					if len(rseq)<(k-1)/2:
						lseq=''
						ll=k-1-len(rseq)
						for l in left:
							if len(lseq)==ll:
								break
							if l in dmap:
								lseq+=dmap[l]
							elif not l=='-':
								lseq+=l.upper()
					kmr=lseq+base+rseq
				if kmr not in kmr_sub:
					kmr_sub[kmr]={strain_sub[strain_set[s]]:''}
				else:
					kmr_sub[kmr][strain_sub[strain_set[s]]]=''
				if kmr not in kmr_pos:
					kmr_pos[kmr]={pos_base:''}
				else:
					kmr_pos[kmr][pos_base]=''
		# recluster according to pos-snp kmer
		#for k in kmr_sub:
		
		return  sub_info,kmr_sub,kmr_pos,sort_sub				
# Load msa
alignment=AlignIO.read(input_msa,'fasta')
#entropy,base_count=estimate_shannon_entropy(alignment[:,7])
sid=range(len(alignment[:,0]))
sname=[]
sname2seq={}  # Strain_Name -> Strain_msa_sequence
cnum=0
for record in alignment:
	name=record.id.strip()
	sname.append('>'+name)	
	sname2seq['>'+name]=str(record.seq)

#sid2name=dict(zip(sid,sname))  # Strain_ID -> Strain_Name
sid2name={}
sname2id=dict(zip(sname,sid))  # Strain_Name -> Strain_ID

## Check how many columns left
## We need to parse the cluster, and exclude used strains
#finished_strains={} # ID -> Strain_Name
strain_info={} # Strain_Name -> Cluster_info
cls_strain={} # Cluster -> Strain_Name -> Strain_ID
cls_rep={}
strain_rep={} # Strain_Name -> Rep_Strain
fc=open(cls_file,'r')
while True:
	line=fc.readline().strip()
	if not line:break
	if re.search('Cluster',line):
		cls=re.sub('>','',line)
		cls_strain[cls]={}
	else:
		ele=line.split('\t')
		if re.search('\*',line):
			rep=ele[0]
			#cls_rep[cls]=rep
			strain_rep[rep]=rep
			strain_info[rep]=cls
			cls_strain[cls][rep]=int(ele[-1])
			sid2name[int(ele[-1])]=rep
			cls_rep[cls]=rep
		else:
			strain_rep[ele[0]]=rep
			strain_info[ele[0]]=cls
			cls_strain[cls][ele[0]]=int(ele[-1])
			sid2name[int(ele[-1])]=ele[0]

### Most important part ----> Split cluster
o1=open('Strain_cls_info.txt','w+')
o2=open('Rebuild_cls.clstr','w+')
o3=open('SubCls_kmer.txt','w+')
raw_cls_size=[]
new_cls_size=[]
for c in cls_strain:
	raw_cls_size.append(len(cls_strain[c]))
	if len(cls_strain[c])==1:
		new_cls_size.append(len(cls_strain[c]))
		for s in cls_strain[c]:
			o1.write(s+'\t'+strain_rep[s]+'\t'+strain_info[s]+'\t'+strain_info[s]+'\t'+str(cls_strain[c][s])+'\t1\n')
			o2.write('>'+c+'\n')
			o2.write('\t'+s+'\t*\t'+str(cls_strain[c][s])+'\n')
	else:
		strain_set={}
		ssname2seq={}
		for s in  cls_strain[c]:
			strain_set[s]=cls_strain[c][s]
			ssname2seq[s]=sname2seq[s]
		sub_info,kmr_sub,kmr_pos,sort_sub=cls_with_perl_hier(ssname2seq,strain_set,src_path)
		if len(sub_info)==0 or len(sort_sub)==1:
			new_cls_size.append(len(cls_strain[c]))
			o2.write('>'+c+'\n')
			for s in cls_strain[c]:
				o1.write(s+'\t'+strain_rep[s]+'\t'+strain_info[s]+'\t'+strain_info[s]+'\t'+str(cls_strain[c][s])+'\t'+str(len(cls_strain[c]))+'\n')
				if s==cls_rep[c]:
					o2.write('\t'+s+'\t*\t'+str(cls_strain[c][s])+'\n')
				else:
					o2.write('\t'+s+'\t'+str(cls_strain[c][s])+'\n')
		else:
			o2.write('>'+c+'\n')
			for sub in sort_sub:
				o2.write('\t>'+sub+'\n')
				new_cls_size.append(len(sub_info[sub]))
				for ss in sub_info[sub]:
					ss=sid2name[ss]
					o1.write(ss+'\t'+strain_rep[ss]+'\t'+strain_info[ss]+'\t'+sub+'\t'+str(cls_strain[c][ss])+'\t'+str(len(sub_info[sub]))+'\n')
					if ss==cls_rep[c]:
						o2.write('\t\t'+ss+'\t*\t'+str(cls_strain[c][ss])+'\n')
					else:
						o2.write('\t\t'+ss+'\t'+str(cls_strain[c][ss])+'\n')
			for kl in kmr_sub:
				info1=','.join(list(kmr_pos[kl].keys()))
				info2=','.join(list(kmr_sub[kl].keys()))
				o3.write(kl+'\t'+c+'\t'+info1+'\t'+info2+'\n')
				seq=Seq(kl,IUPAC.ambiguous_dna)
				k2=seq.reverse_complement()._data
				o3.write(k2+'\t'+c+'\t'+info1+'\t'+info2+'\n')
				
os.system('rm Vbuild_tem.column Vbuild_tem_cls.aln')
plt.hist(raw_cls_size,bins=100)
plt.xlabel('Cluster size')
plt.ylabel('Cluster Number')
plt.savefig('Before_split.png')	
plt.figure()
plt.hist(new_cls_size,bins=100)
plt.xlabel('Cluster size')
plt.ylabel('Cluster Number')
plt.savefig('After_split.png')
### Now we need to iterate all columns
#cls_with_entropy(alignment, centropy, cbase_freq, sid, sname2seq, sid2name, sname2id)



import re
import os,sys
import argparse
import math
from distutils.sysconfig import get_python_lib


__author__="Liao Herui"

usage="VirStrain - An RNA virus strain-level identification tool for short reads."

def main():
	parser=argparse.ArgumentParser(prog='VirStrain_build.py',description=usage)
	parser.add_argument('-i','--input_msa',dest='input_msa',type=str,required=True,help="Input MSA file generated by mafft --- Required")
	parser.add_argument('-d','--database_dir',dest='db_dir',type=str,help='Database Output dir (default: current workdir/VirStrain_DB)')
	parser.add_argument('-c','--dash_cutoff',dest='dash_cutoff',type=str,help='The cutoff of dash in each column of MSA (default: 0)')
	parser.add_argument('-s','--sites_ratio_cutoff',dest='sites_cutoff',type=str,help='The cutoff of sites number for manual-covering function (eg. 1 means all useful sites will be used and 0.8 means 80%% useful sites will be used.)')
	parser.add_argument('-r','--sites_range_cutoff',dest='sites_rcutoff',type=str,help='The cutoff of sites range for covering algorithm (eg. 3-500 means the covering algorithm will only consider the SNV sites from 3-500 of MSA.)')
	
	args=parser.parse_args()	
	in_msa=args.input_msa
	db_dir=args.db_dir
	dashc=args.dash_cutoff
	sitec=args.sites_cutoff
	site_r=args.sites_rcutoff
	
	if not db_dir:
		db_dir='VirStrain_DB'
	if not dashc:
		dashc=0
	if not sitec:
		sitec=0
	else:
		sitec=float(sitec)
	if not site_r:
		site_r=[0,0]
	else:
		siter_t=re.split('-',site_r)
		site_r=[int(siter_t[0]),int(siter_t[1])]
	if not os.path.exists(db_dir):
		os.makedirs(db_dir)
	# Start to build the database
	snum,cnum=scan_msa(in_msa)
	# Run all program
	spk=get_python_lib()+'/VirStrain/'
	# Step1 - Choose sites
	if site_r[1]==0:
		os.system('perl '+spk+'aln2cluster-overlap-kmer-withd.pl '+in_msa+' '+str(snum)+' '+str(dashc)+' 0 '+str(cnum)+' 0 > VirStrain_build.column')
	else:
		os.system('perl '+spk+'aln2cluster-overlap-kmer-withd.pl '+in_msa+' '+str(snum)+' '+str(dashc)+' '+str(site_r[0])+' '+str(site_r[1])+' 0 > VirStrain_build.column')
	if sitec>0:
		manual_covering(sitec,snum,in_msa,site_r,spk)
	# Step2 - Extract kmer and generate snp matrix
	os.system('python '+spk+'S1_extract_kmer.py -i '+in_msa+' -c VirStrain_build.column')
	# Step3 - Divide strains into clusters
	os.system('python '+spk+'S2_remove_redundant_For_Me.py -i Strain_pos_snp_matrix_consider_all.txt')
	# Step4 - Hierarchical Clustering
	os.system('python '+spk+'S2.5_Split_cls_hierarchical_with_perl.py -i '+in_msa+' -r Remove_redundant_matrix_MM_Call.clstr')
	os.system('mv After_split.png Before_split.png ID2Name.txt '+db_dir)
	os.system('mv Pos-snp-kmer-all.fa Pos-snp-kmer-all.txt Rebuild_cls.clstr Remove_redundant_matrix_MM_Call.clstr  Strain_pos_snp_matrix_not_redundant_MM_Call.txt Strain_pos_snp_matrix_consider_all.txt SubCls_kmer.txt Strain_cls_info.txt VirStrain_build.column '+db_dir)
	
def manual_covering(sitec,snum,in_msa,site_r,spk):
	os.system('perl '+spk+'aln2entropy.pl '+in_msa+' '+str(snum)+' > manual.column')
	#print('perl bin/aln2entropy.pl '+in_msa+' '+str(snum)+'> manual.column')
	#exit()
	f=open('manual.column','r')
	o=open('mc.column','w+')
	a=[]
	while True:
		line=f.readline().strip()
		if not line:break
		if not re.search('column',line):continue
		ele=line.split()
		cid=int(ele[1])
		for e in ele:
			if re.search('-',e):
				dash_num=re.sub('-:','',e)
				break
		dash_num=int(dash_num)
		if dash_num==0:
			if not site_r[1]==0:
				if cid>=site_r[0] and cid<=site_r[1]:
					a.append(line)
			else:
				a.append(line)
				#o.write(line+'\n')
	out_c=math.ceil(sitec*float(len(a)))
	#out_c=int(out_c)
	outa=a[:out_c]
	print('Newly added sites number: ',len(outa),'/',len(a))
	for line in outa:
		o.write(line+'\n')
	o.close()
	os.system('rm manual.column')
	os.system('cat VirStrain_build.column mc.column > VirStrain_build_tem.column')
	os.system('rm VirStrain_build.column mc.column')
	os.system('mv VirStrain_build_tem.column VirStrain_build.column')

def scan_msa(in_msa):
	fs1=open(in_msa,'r')
	dseq={}
	while True:
		line=fs1.readline().strip()
		if not line:break
		if re.search('>',line):
			name=line
			dseq[name]=''
		else:
			dseq[name]+=line
	cnum=0
	for d in dseq:
		cnum=len(dseq[d])
		break
	return len(dseq),cnum

if __name__=='__main__':
	sys.exit(main())



import re
import os
import getopt
import sys
import networkx
from networkx.algorithms.components.connected import connected_components

input_matrix=''

opts,args=getopt.getopt(sys.argv[1:],"i:")
for opt,arg in opts:
	if opt=='-i':
		input_matrix=arg

f=open(input_matrix,'r')
o1=open('Strain_pos_snp_matrix_not_redundant_MM_Call.txt','w+')
o2=open('Remove_redundant_matrix_MM_Call.clstr','w+')
line=f.readline().strip()
o1.write('\t\t'+line+'\n')
head_line=line.split('\t')
carr=[]
base=['A','T','G','C']
for e in head_line:
	c=re.split('-',e)[0]
	if int(c) not in carr:
		carr.append(int(c))
#o1.write('\t\t'+line+'\n')

arr=[]
ds_snp={}
ds2id={}
#strain=[]
i=0
while True:
	line=f.readline().strip()
	if not line:break
	ele=line.split('\t')
	tem=[]
	tem.append(ele[0])
	ds2id[ele[0]]=i
	i+=1
	#strain.append(ele[0])
	ds_snp[ele[0]]=ele[1:]
	snp_info=','.join(ele[1:])
	tem.append(snp_info)
	arr.append(tem)

def to_graph(l):
	G=networkx.Graph()
	for part in l:
		G.add_nodes_from(part)
		G.add_edges_from(to_edges(part))
	return G

def to_edges(l):
	it=iter(l)
	last=next(it)

	for current in it:
		yield last,current
		last=current

G=to_graph(arr)
i=1
final_strain=[]
dcsb={} # column -> strain -> base
for c in carr:
	dcsb[c]={}
for ele in list(connected_components(G)):
	ele=list(ele)
	final=[]
	for e in ele:
		if not re.search(',',e):
			final.append(e)
	o2.write('>Cluster'+str(i)+'_'+str(len(final))+'\n')
	i+=1
	o2.write('\t'+final[0]+'\t*\t'+str(ds2id[final[0]])+'\n')
	o1.write(final[0]+'\t'+'\t'.join(ds_snp[final[0]])+'\n')
	final_strain.append(final[0])
	ti=0
	for e in ds_snp[final[0]]:
		if int(e)==1:
			info=head_line[ti]
			arr=re.split('-',info)
			dcsb[int(arr[0])][final[0]]=arr[1]
		ti+=1
	for s in final[1:]:
		o2.write('\t'+s+'\t'+str(ds2id[s])+'\n')
'''
o1.write('POS,')
s1=','.join(final_strain)
o1.write(s1+'\n')
print(len(final_strain))
exit()
for c in carr:
	o1.write(str(c)+',')
	tem=[]
	for s in final_strain:
		tem.append(dcsb[c][s])
	s2=','.join(tem)
	o1.write(s2+'\n')
'''



import re
import os
import getopt
import sys
#from collections import Counter
#from Bio.Seq import Seq
#from Bio.Alphabet import IUPAC
import numpy as np
#import pandas as pd
### Dehan Method Check
#from scipy import sparse
#from sklearn.model_selection import ShuffleSplit
#from sklearn.linear_model import Lasso, LassoCV
#from scipy.stats import pearsonr

opts,args=getopt.getopt(sys.argv[1:],"i:p:k:m:s:f:c:b:o:")
read_1=''	# SE or long reads
read_2=''	# PE reads - read set 2
snp_kmr_file='' #kmer -> snp_pos file
snp_kmr_fa='' # kmr fasta
matrix_file='' # Sno-pos matrix file
cls_file=''
sub_kmr_file=''
out_dir=''

k=25		# Default k size=25
#min_depth_rate=0.1
min_depth_percentile=10
max_depth_percentile=90
min_depth_absolute=2
#min_depth_rate=0.05
min_depth_rate=0.05
#threads=4
for opt,arg in opts:
	if opt=='-i':
		read_1=arg
	elif opt=='-p':
		read_2=arg
	elif opt=='-k':
		k=int(arg)
	elif opt=='-s':
		snp_kmr_file=arg
	elif opt=='-m':
		matrix_file=arg
	elif opt=='-f':
		snp_kmr_fa=arg
	elif opt=='-c':
		cls_file=arg
	elif opt=='-b':
		sub_kmr_file=arg
	elif opt=='-o':
		out_dir=arg

## Static variables
BASE_ORDER=['A','T','G','C']
BASE_P = {'A': [1, 0, 0, 0],'C':[0,1,0,0],'G':[0,0,1,0],'T':[0,0,0,1],}
'''
CV_NITER = 20
NALPHA = 50
MAX_NITER = 5000
TEST_SIZE = 0.5
'''
file_dir=sys.path[0]
#print(file_dir)
#exit()


######### Step-1 Load Pre-build File to memory ####
## Kmer -> POS-SNP
f1=open(snp_kmr_file,'r')
dkps={}  # kmr -> {pos-snp:1,......}
pos=[]
dpsc={} # pos-snp:  num
while True:
	line=f1.readline().strip()
	if not line:break
	ele=line.split('\t')
	dkps[ele[0]]=''
	ps=re.split(',',ele[1])
	for e in ps:
		dkps[ele[0]]=e # Set to 1 for Counter -> Dict Merge
		dpsc[e]=0

## Build pos-snp freq array

f3=open(matrix_file,'r')
fl=f3.readline().strip()
pos_snp=re.split('\t',fl) # Head line arr
#print(np.where(np.array(pos_snp)=='8946-T')[0][0])

# Run jellyfish to get kmer counting result
if read_2=='':
	cmd1=file_dir+'/jellyfish-linux count -m 25 -s 100M -t 8 --if '+snp_kmr_fa+' -o Tem_VS.jf '+read_1
	cmd2=file_dir+'/jellyfish-linux dump -c Tem_VS.jf > Tem_Vs.fa'
	os.system(cmd1)
	os.system(cmd2)
else:
	cmd1=file_dir+'/jellyfish-linux count -m 25 -s 100M -t 8 --if '+snp_kmr_fa+' -o Tem_VS.jf '+read_1+' '+read_2
	cmd2=file_dir+'/jellyfish-linux dump -c Tem_VS.jf > Tem_Vs.fa'
	os.system(cmd1)
	os.system(cmd2)
	

freq_arr=[]
fnew=open('Tem_Vs.fa','r')
while True:
	line=fnew.readline().strip()
	if not line:break
	ele=line.split()
	dpsc[dkps[ele[0]]]+=int(ele[1])

carr=[]
for p in pos_snp:
	c=re.split('-',p)[0]
	if c not in carr:
		carr.append(c)
	if p not in dpsc:
		freq_arr.append(0)
	else:
		freq_arr.append(dpsc[p])

freq_arr=np.array(freq_arr)
#print(freq_arr[1397])

#freq_arr[freq_arr<=min_depth_absolute]=0
#keep=(freq_arr!=0)


### Check avg depth from the pos-snp frequency array
keep=(freq_arr!=0)
check_arr=freq_arr[keep]
min_depth,max_depth=np.percentile(check_arr,[min_depth_percentile,max_depth_percentile])
#print(min_depth)
keep=np.logical_and.reduce((check_arr>=min_depth,check_arr<=max_depth))
check_arr2=check_arr[keep]
# Average depth of the frequency vector
min_depth_adf=min_depth_rate*np.mean(check_arr2)
##### !! freq arr filter using 2 or 0.1*avg-depth

if min_depth_adf<2:
	min_depth_adf=2 # Use 2 to test firstly
#min_depth_adf=min_depth_absolute
freq_arr[freq_arr<=min_depth_adf]=0
weighted_freq_arr=freq_arr/np.sum(freq_arr)

##### Filter done. ########


pos_freq_map=dict(zip(pos_snp,freq_arr))
#print(pos_freq_map['18722-C'])
t=np.array(pos_snp)
#sindex=np.argwhere(t=='18722-C')[0][0]
#print(freq_arr[sindex])
#exit()
#print(np.where(np.array(pos_snp)=='8946-T'))
	#ax=sns.distplot(a,norm_hist=False,kde=False)
	#ax=sns.distplot(a,norm_hist=False,kde=False)
#exit()

#print(pos_freq_map['8946-T'])
#exit()
#print(pos_freq_map['4248-T'])
#exit()
#des={}  # Strain -> the sum of frequency vector of this strain
ds_pos={}  # Strain -> 0 1 1 0 0 1 (pos-snp: yes or no vector)
ds_freq={} # Strain -> frequency vector of this strain
dmap_rate={} # Strain -> pos-snp map scpre of this strain
#ds_avgd={}  # Strain -> average depth of this strain
ds_num={} # Strain -> pos-snp map number, raw number
dmr={}  # Strain -> pos-snp map rate os this strain (map_number/raw_number)
#These 2 dict will be used to visualize the pos depth figure
dscf={} #Strain-> 110-A:110, 111-NA:0, ..
dscl={} # Strain-> 110-A:3000, 111-NA:0, ..
all_ps=[] # Record the matrix array
while True:
	line=f3.readline().strip()
	if not line:break
	ele=line.split('\t')
	#des[ele[0]]=0
	tem=[]
	#all_ps.append(ele[1:])
	for e in ele[1:]:
		tem.append(int(e))
	tem=np.array(tem)
	all_ps.append(tem)
	dscf[ele[0]]={}
	dscl[ele[0]]={}
	'''
	keep=(tem==1)
	ps_number=len(tem[keep])
	ds_num[ele[0]]=ps_number
	'''
	nt=freq_arr*tem # the frequency vector of this strain
	raw_c=len(tem[tem==1])
	map_c=len(nt[nt>0])
	'''
	if ele[0]=='>MT419847.1':
		print('>MT419847.1',map_c,raw_c)
	if ele[0]=='>MN938384.1':
		print('>MN938384.1',map_c,raw_c)
	'''
	map_rate=np.sum(tem*weighted_freq_arr)
	#map_rate=float(map_c)/float(raw_c)
	#dmap_rate[ele[0]]=map_rate
	dmap_rate[ele[0]]=map_rate
	ds_num[ele[0]]=str(map_c)+'/'+str(raw_c)
	dmr[ele[0]]=float(map_c)/float(raw_c)
	#value=nt.sum()
	#des[ele[0]]=value
	ds_pos[ele[0]]=tem
	ds_freq[ele[0]]=nt
	## Get average depth of this strain ##
	'''
	keep=(nt!=0)
	nt=nt[keep]
	min_depth,max_depth=np.percentile(nt,[min_depth_percentile,max_depth_percentile])
	keep=np.logical_and.reduce((nt>=min_depth,nt<=max_depth))
	nt=nt[keep]
	ds_avgd[ele[0]]=np.mean(nt)
	'''
''' Here we initialize 2 dict for later visualization. '''
all_ps=np.array(all_ps)
all_sum=np.sum(all_ps,axis=0)
pos_label=dict(zip(pos_snp,list(all_sum)))
'''
for s in ds_freq:
	i=0
	for c in ds_freq[s]:
		if c==0:
			i+=1
			continue
		current_ps=pos_snp[i]	
		column=int(re.split('-',current_ps)[0])
		if column not in dscf[s]:
			dscf[s][column]=0
		if not c==0:
			dscf[s][column]+=c
		i+=1
	i2=0
	for c in ds_pos[s]:
		if c==0:
			i2+=1
			continue
		current_ps=pos_snp[i2]
		column=int(re.split('-',current_ps)[0])
		if column not in dscl[s]:
			dscl[s][column]=0
		if c==1:
			dscl[s][column]=pos_label[current_ps]
		i2+=1
print(len(dscl['>gb:J02176']),len(dscf['>gb:J02176']))
exit()
'''
#print(max(the_sum),len(np.argwhere(the_sum==len(all_ps_vec)-1)),len(np.argwhere(the_sum==1)))

#exit()
#index=np.where(np.array(pos_snp)=='8946-T')[0][0]
#print(ds_pos['>MT419847.1'][index])
#print(ds_pos['>MN938384.1'][index])
max_map=sorted(dmr.items(),key=lambda d:d[1],reverse=True)[0][1]
#exit()
res=sorted(dmap_rate.items(),key=lambda d:d[1],reverse=True)
top10_score_s=res[:10]
#exit()
top_map_strain=[]
for r in res:
	if r[1]==res[0][1]:
		top_map_strain.append(r[0])
	else:break
#####  Unique nodes scan for all top strains
snp_arr=[]
### Pre-calculate the possible strain number, then decide whether should calculate weighted score.
pre_freq_arr=[]
#strain_num={}
for s in top_map_strain:
	snp_arr.append(ds_pos[s])
	pre_pa=ds_pos[s]*(-1)
	pre_pa=np.array(pre_pa)
	pre_pa[pre_pa==0]=1
	pre_freq_arr=freq_arr*pre_pa
	pre_freq_arr[pre_freq_arr<0]=0

keep=(pre_freq_arr!=0)
pre_freq_arr=pre_freq_arr[keep]

pre_pos_snp=np.array(pos_snp)[keep]
pre_ds_pos={}
for s in ds_pos:
	pre_ds_pos[s]=ds_pos[s][keep]
pre_wf_arr=pre_freq_arr/np.sum(pre_freq_arr)
strain_num={}
sn=0
#print(pre_freq_arr)
#exit()
while True:
	if len(pre_freq_arr)==0:break
	smr={}
	for r in ds_pos:
		if r in top_map_strain:continue
		tt=pre_ds_pos[r]
		nt=tt*pre_wf_arr
		#mc=len(nt[nt>0])
		mr=np.sum(nt)
		smr[r]=mr
	res=sorted(smr.items(),key=lambda d:d[1],reverse=True)
	ts=[]
	for r in res:
		if r[1]==res[0][1]:
			ts.append(r[0])
	if len(ts)>1:
		rmr={}
		for s in ts:
			rmr[s]=dmap_rate[s]
		res2=sorted(rmr.items(),key=lambda d:d[1],reverse=True)
		#for r in res2:
		strain_num[res2[0][0]]=''
	else:
		strain_num[ts[0]]=''
	#strain_num[res[0][0]]=''
	vm1=len(pre_freq_arr)
	pre_pa=pre_ds_pos[ts[0]]*(-1)
	pre_pa[pre_pa==0]=1
	pre_freq_arr=pre_freq_arr*pre_pa
	pre_freq_arr[pre_freq_arr<0]=0
	keep=(pre_freq_arr!=0)
	pre_freq_arr=pre_freq_arr[keep]
	if not np.sum(pre_freq_arr)==0:
		pre_wf_arr=pre_freq_arr/np.sum(pre_freq_arr)
	pre_pos_snp=pre_pos_snp[keep]
	for s in pre_ds_pos:
		pre_ds_pos[s]=pre_ds_pos[s][keep]
	vm=vm1-len(pre_freq_arr)
	if vm>1:
		#strain_num[res[0][0]]=''
		sn+=1
# Will recalculate the score and select top strain
if sn>1:
	#nw=len(strain_num)
	for s in top_map_strain:
		strain_num[s]=''
	sscore={}
	sna=[]
	for s in strain_num:
		sna.append(ds_pos[s])
	sna=np.array(sna)
	ssum=sna.sum(axis=0)
	ssum[ssum==0]=1
	for s in strain_num:
		snt=ds_pos[s]/ssum
		ns=dmap_rate[s]*snt
		ns=ns.sum(axis=0)
		sscore[s]=ns
	res=sorted(sscore.items(),key=lambda d:d[1],reverse=True)
	tem_map_strain=[]
	for r in res:
		if not dmr[r[0]]==max_map:continue
		tem_map_strain.append(r[0])
		break
	if len(tem_map_strain)>0:
		top_map_strain=tem_map_strain
	#print(sscore)

	
snp_arr=np.array(snp_arr)
pos_sum=snp_arr.sum(axis=0)
pos_sum[pos_sum>1]=0
i=0

strain_unique={}
strain_unique_count={}
for p in pos_sum:
	column=pos_snp[i]
	#i+=1
	if p==1:
		if pos_freq_map[column]<=min_depth_absolute:
			i+=1
			continue
		i2=0
		window=snp_arr[:,i]
		for w in window:
			if w==1:
				strain=top_map_strain[i2]
				if strain not in strain_unique:
					strain_unique[strain]={column:pos_freq_map[column]}
					strain_unique_count[strain]=1
				else:
					strain_unique[strain][column]=pos_freq_map[column]
					strain_unique_count[strain]+=1
			i2+=1
	i+=1
#print(freq_arr[1397])
#print(pos_freq_map['4248-T'])
#print(strain_unique)
#exit()
## Final output generate part ####
mp_strain=[]  # Most possible strain
op_strain=[]  # Other possible strain -> [S1,S2,S3,S5]
op_strain_batch=[] # Other possible strain ->[[S1,S2],[S3,S5],...]
#op_pos_snp=[] # Other possible pos-snp
#op_ps_strain=[] # The top map rate strain of other possible pos-snp
if not len(strain_unique)==0:
	#print(strain_unique)
	for s in strain_unique:
		mp_strain.append(s)
		#print(s,ds_avgd[s])
		#exit()
else:
	### Need to check whether these strains have close depth
	#d={}
	#strain_depth=[]
	for r in top_map_strain:
		mp_strain.append(r)
		#d[r]=ds_avgd[r]
		#raw_depth=ds_freq[r] # the freq arr of this strain -> [0, 22, 0 ...]
		#keep=(raw_depth>0)
		#nz_depth=raw_depth[keep]
		#print(np.mean(nz_depth))
		#exit()
	#exit()
	'''
	res2=sorted(d.items(),key=lambda d:d[1],reverse=True)
	for r in res2:
		mp_strain.append(r[0])
	'''
#print(strain_unique)
#exit()
#print(top_map_strain)
#for t in top_map_strain:
#print(t,dmap_rate[t])
## Check nodes of MT419847
## Iterative function to get other possible strains and other possible pos-snp
#print('Raw freq arr: ', freq_arr[sindex])
ds_avgd={}
for m in mp_strain:
	'''
	keep=(ds_freq[m]!=0)
	check_arr=ds_freq[m][keep]
	min_depth,max_depth=np.percentile(check_arr,[min_depth_percentile,max_depth_percentile])
	kp1=(check_arr<=min_depth)
	check_arr=check_arr[kp1]
	continue
	print(m,ds_freq[m][sindex])
	'''
	# Get the avg depth of the strain
	keep=(ds_freq[m]!=0)
	min_depth,max_depth=np.percentile(ds_freq[m][keep],[min_depth_percentile,max_depth_percentile])
	keep=np.logical_and.reduce((ds_freq[m]>=min_depth,ds_freq[m]<=max_depth))
	ds_avgd[m]=np.mean(ds_freq[m][keep])

	pos_arr=ds_pos[m]*(-1)
	pos_arr=np.array(pos_arr)
	pos_arr[pos_arr==0]=1
	freq_arr=freq_arr*pos_arr
	freq_arr[freq_arr<0]=0

keep=(freq_arr!=0)

left_freq_arr=freq_arr[keep]
#print('Filtered freq arr: ',left_freq_arr[sindex])
#exit()
pos_snp=np.array(pos_snp)
left_pos_snp=pos_snp[keep]
left_ds_pos={}
for s in ds_pos:
	left_ds_pos[s]=ds_pos[s][keep]
#print(left_freq_arr)
#print(left_pos_snp)
#exit()
#run=0
left_ps_freq_map=dict(zip(left_pos_snp,left_freq_arr))
#print(left_ps_freq_map)
#exit()
resl=sorted(left_ps_freq_map.items(),key=lambda d:d[1],reverse=True)
os_strain={} # '221-A-100-10000':['>MT312312.1',....]
os_arr=[] # ['221-A-100-10000','225-G-100-10000',....]
left_weighted_freq_arr=left_freq_arr/np.sum(left_freq_arr)
#print(left_ps_freq_map)
#print(left_freq_arr,left_pos_snp)
#exit()
#run=0
vmap={} # The dict used to record the valid map rate
#### Start Iterative process ######
if not len(left_freq_arr)==0:
	max_iter_times=len(left_freq_arr)
	for l in range(max_iter_times):
		#temd=dict(zip(left_pos_snp,left_freq_arr))
		#res=sorted(temd.items(),key=lambda d:d[1],reverse=True)
		#for r in res:
		if len(left_freq_arr)==0:break
		#column_snp=res[0][0]
		strain_map_rate={}
		for r in ds_pos:
			if r in mp_strain:continue
			#if r in op_strain:continue
			#if r in op_ps_strain:continue
			tem=left_ds_pos[r]
			nt=tem*left_weighted_freq_arr
			#raw_c=len(tem[tem==1])
			map_c=len(nt[nt>0])
			#map_rate=map_c
			map_rate=np.sum(nt)
			strain_map_rate[r]=map_rate
		selected_strain=[] # used to save the selected strains in this round
		res=sorted(strain_map_rate.items(),key=lambda d:d[1],reverse=True)
		#print(res)
		#exit()
		top_s=[]
		snp_arr=[]

		for r in res:
			if r[1]==res[0][1]:
				top_s.append(r[0])
				### Calculate avg depth
				nt=left_ds_pos[r[0]]*left_freq_arr
				keep=(nt!=0)
				nt=nt[keep]
				min_depth,max_depth=np.percentile(nt,[min_depth_percentile,max_depth_percentile])
				keep=np.logical_and.reduce((nt>=min_depth,nt<=max_depth))
				if not len(nt[keep])==0:
					nt=nt[keep]
				ds_avgd[r[0]]=np.mean(nt)
				## Done
				snp_arr.append(left_ds_pos[r[0]])
		#print(top_s)
		#exit()
		if len(top_s)>1:
			rank_map_rate={}
			for s in top_s:
				rank_map_rate[s]=dmap_rate[s]
			res=sorted(rank_map_rate.items(),key=lambda d:d[1],reverse=True)
			top_s=[]
			for r in res:
				top_s.append(r[0])
		top_s=np.array(top_s)
		pre=[]
		for r in resl:
			pre.append(r[0]+':'+str(r[1]))
		pre=','.join(pre)
		top_pos_snp=pre+'\t'+str(len(top_s))
		os_arr.append(top_pos_snp)
		os_strain[top_pos_snp]=[]
		for s in top_s:
			os_strain[top_pos_snp].append(s)
			pos_arr=left_ds_pos[s]*(-1)
			pos_arr[pos_arr==0]=1
			left_freq_arr=left_freq_arr*pos_arr
			left_freq_arr[left_freq_arr<0]=0
		keep=(left_freq_arr!=0)
		valid_map=len(left_freq_arr)
		left_freq_arr=left_freq_arr[keep]
		valid_map=valid_map-len(left_freq_arr)
		vmap[top_pos_snp]=valid_map
		if not np.sum(left_freq_arr)==0:
			left_weighted_freq_arr=left_freq_arr/np.sum(left_freq_arr)
		left_pos_snp=left_pos_snp[keep]
		left_ps_freq_map=dict(zip(left_pos_snp,left_freq_arr))
		for s in left_ds_pos:
			left_ds_pos[s]=left_ds_pos[s][keep]
		resl=sorted(left_ps_freq_map.items(),key=lambda d:d[1],reverse=True)

# All strain cross validation

### Output part
'''
fp=open('../head_file.txt','r')
dc={}
while True:
	line=fp.readline().strip()
	if not line:break
	pre=re.split('\|',line)[0].strip()
	anno=re.split('\|',line)[1].strip()
	#t=re.split('/',line)
	if len(t)<3:
		dc[pre]='NA'
	else:
	#country=re.split('/',line)[2]
	#if re.search('2020',country):
	#country='NA'
	dc[pre]=anno
'''
o=open(out_dir,'w+')
o.write('\t\tStrain_ID\tCls_info\tSubCls_info\tMap_Score\tValid_Map_Rate\tTotal_Map_Rate\tStrain_Depth\tStrain_info\tUnique_SNP\n')
o.write('>>Most possible strains:\n')
all_s=[]
vs_sd=[]
vs_so=[]
for s in mp_strain:
	all_s.append(s)
	vs_sd.append(s)
if len(os_arr)>0:
	for s in os_arr:
		all_s.append(os_strain[s][0])
		if vmap[s]>1:
			vs_so.append(os_strain[s][0])
# Strain-level identification 
s2cls={} # Used for output cluster info
s2sub={} # Used for output sub-cluster info
candidate_cls={}
fcls=open(cls_file)
while True:
	line=fcls.readline().strip()
	if not line:break
	ele=line.split('\t')
	if ele[0] not in all_s:
		s2cls[ele[0]]=ele[2]
		continue
	if ele[2]==ele[3]:
		s2cls[ele[0]]=ele[2]
		s2sub[ele[0]]='NA'
	else:
		s2cls[ele[0]]=ele[2]
		candidate_cls[ele[2]]=''
		#s2sub[ele[0]]=''
if not len(candidate_cls)==0:
	fsk=open(sub_kmr_file,'r')
	ksub={}
	cls_sub={}
	#subcount={}
	while True:
		line=fsk.readline().strip()
		if not line:break
		ele=line.split('\t')
		if ele[1] in candidate_cls:
			if k not in ksub:
				ksub[k]={ele[1]:{}}
			if ele[1] not in ksub[k]:
				ksub[k][ele[1]]={}
			if ele[1] not in cls_sub:
				cls_sub[ele[1]]={}
			sub=re.split(',',ele[-1])
			for s in sub:
				ksub[k][ele[-1]][s]=''
				cls_sub[ele[1]][s]=0
	ok=open('Tem_Vs2Sub.fa','w+')
	co=1
	for k in ksub:
		ok.write('>'+str(co)+'\n')
		ok.write(k+'\n')
		co+=1
	ok.close()
	if read_2=='':
		cmd1=file_dir+'/jellyfish-linux count -m 25 -s 100M -t 8 --if Tem_Vs2Sub.fa -o Tem_VS2.jf '+read_1
	else:
		cmd1=file_dir+'/jellyfish-linux count -m 25 -s 100M -t 8 --if Tem_Vs2Sub.fa -o Tem_VS2.jf '+read_1+' '+read_2
	cmd2=file_dir+'/jellyfish-linux dump -c Tem_VS2.jf > Tem_Vs2.fa'
	ft2=open('Tem_Vs2.fa','r')
	while True:
		line=ft2.readline().strip()
		if not line:break
		ele=line.split()
		if int(ele[1])>=min_depth_adf:
			for c in ksub[ele[0]]:
				for c2 in ksub[ele[0]][c]:
					cls_sub[c][c2]+=int(ele[1])
	for s in all_s:
		if s in s2sub:continue
		if s2cls[s] not in cls_sub:
			s2sub[s]='NA'
		else:
			res=sorted(cls_sub[s2cls[s]].items(),key=lambda d:d[1],reverse=True)
			if res[0][1]==res[1][1]:
				s2sub[s]='NA'
			else:
				s2sub[s]=res[0][0]


for s in mp_strain:
	#keep=(ds_pos[s]==1)
	#all_s.append(s)
	if s in strain_unique:
		o.write('\t\t'+s+'\t'+s2cls[s]+'\t'+s2sub[s]+'\t'+str(dmap_rate[s])+'\t'+str(ds_num[s])+'\t'+str(ds_num[s])+'\t'+str(ds_avgd[s])+'\t\t\t'+str(strain_unique[s])+'\n')
	else:
		o.write('\t\t'+s+'\t'+s2cls[s]+'\t'+s2sub[s]+'\t'+str(dmap_rate[s])+'/'+str(ds_num[s])+'\t'+str(ds_avgd[s])+'\t\t\tNA\n')
o.write('>>Other possible strains:\n')
if len(os_arr)>0:
	#i=1
	for s in os_arr:
		ele=re.split('\t',s)
		if vmap[s]==1:
			o.write('\t>>(Could be FP)'+ele[0]+',Genome_num: '+ele[1]+'\n')
		else:
			o.write('\t>>'+ele[0]+',Genome_num: '+ele[1]+'\n')
		all_s.append(os_strain[s][0])
		for n in os_strain[s]:
			#all_s.append(n)
			a=re.split('/',ds_num[n])[-1]
			vm=str(vmap[s])+'/'+a
			if n in s2sub:
				o.write('\t\t'+n+'\t'+s2cls[n]+'\t'+s2sub[n]+'\t'+str(dmap_rate[n])+'\t'+vm+'\t'+ds_num[n]+'\t'+str(ds_avgd[n])+'\t\t\tNot_record\n')
			else:
				o.write('\t\t'+n+'\t'+s2cls[n]+'\tNot_record\t'+str(dmap_rate[n])+'\t'+vm+'\t'+ds_num[n]+'\t'+str(ds_avgd[n])+'\t\t\tNot_record\n')
				
		#outs='\t'.join(os_strain[s])
		#o.write('\t\t\t'+outs+'\n')
		#i+=1
else:
	o.write('\tCan not detect other strains.\n')
res=sorted(dmr.items(),key=lambda d:d[1],reverse=True)
o.write('\n>>Highest_Map_Strains (Could be FP):\n')
final={}
for r in res:
	if r[1]==res[0][1]:
		if r[0] not in all_s:
			final[r[0]]=dmap_rate[r[0]]
if not len(final)==0:
	res=sorted(final.items(),key=lambda d:d[1],reverse=True)
	for s in res:
		if s[0] in s2sub:
			o.write('\t\t'+s[0]+'\t'+s2cls[s[0]]+'\t'+s2sub[s[0]]+'\t'+str(dmap_rate[s[0]])+'\t'+ds_num[s[0]]+'\t'+ds_num[s[0]]+'\tNA\tNA\n')
		else:
			o.write('\t\t'+s[0]+'\t'+s2cls[s[0]]+'\tNot_record\t'+str(dmap_rate[s[0]])+'\t'+ds_num[s[0]]+'\t'+ds_num[s[0]]+'\tNA\tNA\n')
o.write('>>Top10_Score_Strains:\n')
for t in top10_score_s:
	if t[0] in s2sub:
		o.write('\t\t'+t[0]+'\t'+s2cls[t[0]]+'\t'+s2sub[t[0]]+'\t'+str(t[1])+'\t'+ds_num[t[0]]+'\t'+ds_num[t[0]]+'\tNA\tNA\n')
	else:
		o.write('\t\t'+t[0]+'\t'+s2cls[t[0]]+'\tNot_record\t'+str(t[1])+'\t'+ds_num[t[0]]+'\t'+ds_num[t[0]]+'\tNA\tNA\n')
## Remove tem file 
os.system('rm Tem_Vs* Tem_VS*')
## From this line, we will generate strain-level analysis report
print('Txt report is done. Now will generate pdf report!')

vs_so=vs_so[:5]
for s in ds_freq:
	check=0
	if s  in vs_sd:
		check=1
	if s in vs_so:
		check=2
	if check==0:continue
	i=0
	for c in ds_freq[s]:
		if c==0:
			i+=1
			continue
		current_ps=pos_snp[i]
		column=int(re.split('-',current_ps)[0])
		if column not in dscf[s]:
			dscf[s][column]=c
		
		i+=1
	i2=0
	for c in ds_pos[s]:
		if c==0:
			i2+=1
			continue
		current_ps=pos_snp[i2]
		column=int(re.split('-',current_ps)[0])
		if column not in dscl[s]:
			dscl[s][column]=0
		if c==1:
			dscl[s][column]=pos_label[current_ps]
		i2+=1

ov1=open('Mps_ps_depth.csv','w+')
ov2=open('Ops_ps_depth.csv','w+')
ov1.write('ID,Column_ID')
#carr=[]
for s in vs_sd:
	ov1.write(','+s+'_Freq')
	ov1.write(','+s+'_LNum')
ov1.write('\n')
carr=sorted(list(dscf[vs_sd[0]].keys()))
i=1
for c in carr:
	ov1.write(str(i)+','+str(c))
	for s in vs_sd:
		ov1.write(','+str(dscf[s][c])+','+str(dscl[s][c]))
	ov1.write('\n')
	i+=1
	
	

ov2.write('ID,Column_ID')
if len(vs_so)==0:
	ov2.write(',None,None\n')
else:
	for s in vs_so:
		ov2.write(','+s+'_Freq')
		ov2.write(','+s+'_LNum')
	ov2.write('\n')
	i=1
	for c in carr:
		ov2.write(str(i)+','+str(c))
		for s in vs_so:
			ov2.write(','+str(dscf[s][c])+','+str(dscl[s][c]))
		ov2.write('\n')
		i+=1



import setuptools

setuptools.setup(
	name="virstrain",
	version="1.13",
	author="Liao Herui",
	author_email="heruiliao2-c@my.cityu.edu.hk",
	description="An RNA/DNA virus strain-level identification tool for short reads.",
	long_description="VirStrain can be applied to identify RNA/DNA viral strains from short reads.",
	long_description_content_type="text/markdown",
	url="https://github.com/liaoherui/VirStrain",
	packages=setuptools.find_packages(),
	include_package_data=True,
	zip_safe=True,
	package_data={
		"VirStrain":['jellyfish-linux',"*.pl"]
	},
	install_requires=[
	'networkx==2.4',
	'numpy==1.17.3',
	'pandas==1.0.1',
	'biopython==1.74',
	'plotly==3.10.0',
	'matplotlib==3.1.2'
	],
	entry_points={
		'console_scripts':[
		'virstrain = VirStrain.VirStrain:main',
		'virstrain_build = VirStrain.VirStrain_build:main',
		]
		},
	classifiers=[
		"Programming Language :: Python :: 3",
		],
)



import re
import os
import getopt
import sys
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC

input_msa=''
k=25  # default kmer size
cfile='' # Column file generated by perl file from Prof Sun and transformed by my code
out='Pos-snp-kmer-all.txt' # default
out2='Pos-snp-kmer-all.fa' # default

opts,args=getopt.getopt(sys.argv[1:],"i:k:c:o:f:")

for opt,arg in opts:
	if opt=='-i':
		input_msa=arg
	elif opt=='-k':
		k=int(arg)
	elif opt=='-c':
		cfile=arg
	elif opt=='-o':
		out=arg
	elif opt=='-f':
		out2=arg


dmap={'a':'A','t':'T','g':'G','c':'C','A':'A','T':'T','G':'G','C':'C'}
check_dict={'a':'A','t':'T','g':'G','c':'C','A':'A','T':'T','G':'G','C':'C','-':''}

f1=open(cfile,'r')
dseq={}
column_arr=[]
#dpbk={} #dict -> pos_base_kmr
while True:
	line=f1.readline().strip()
	if not line:break
	if not re.search('column',line):continue
	ele=line.split()
	'''
	if len(ele)<3:
		print(ele)
		exit()
	'''
	if int(ele[1]) not in column_arr:
		column_arr.append(int(ele[1]))
	'''
	#if ele[-2] not in dpbk:
		#dpbk[ele[-2]]={dmap[ele[-1]]:{}}
	#else:
		#if dmap[ele[-1]] in dpbk[ele[-2]]:
			#print('Double Column SNP - Check plz.')
			#exit()
		#dpbk[ele[-2]][dmap[ele[-1]]]={}
	for e in ele[1:-2]:
		if re.search(':',e):
			name=re.split(':',e)[0]
		else:
			name=e
		dseq[name]=''
	'''
#print(len(column_arr))
column_arr=sorted(column_arr)

f2=open(input_msa,'r')
dmsa={}
ds2id={}   # Strain_Name -> Strain_ID
did2s={}
oi=open('ID2Name.txt','w+')
i=0
while True:
	line=f2.readline().strip()
	if not line:break
	if re.search('>',line):
		name=line.split()[0].strip()
		#if name not in dseq:continue
		dmsa[name]=''
		oi.write(str(i)+'\t'+name+'\n')
		ds2id[name]=str(i)
		did2s[i]=name
		i+=1
	else:
		#if name not in dseq:continue
		dmsa[name]+=line
'''
ot=open('Target_msa.fasta','w+')
target=[3344,0,9547,9548,9549,9550]
for t in target:
	ot.write('>'+did2s[t]+'\n')
	s=''
	for c in column_arr:
		s+=dmsa[did2s[t]][c]
	ot.write(s+'\n')
exit()
'''
### Most important part ###
dpbk={} # Dict -> Pos:Base:Kmr
dkpb={}	# Dict -> Kmr: POS-Base
dkl={}  # Dict -> Kmr : Label
dks={}  # Dict -> kmr : Strain
total_column=len(column_arr)
i=1
for c in column_arr:
	print('Progress: ',i,'/',total_column)
	i+=1
	sid=0
	for s in dmsa:
		if dmsa[s][c] not in dmap:continue
		base=dmap[dmsa[s][c]]
		left=dmsa[s][:c][::-1]  # Left Sequence of column base
		right=dmsa[s][c+1:]	    # Right Sequence of column base
		pos_base=str(c)+'-'+base
		block_seq=dmsa[s][c-12:c]+base+dmsa[s][c+1:c+13]
		check_num=0
		for b in block_seq:
			if b not in check_dict:
				check_num=1
				break
		if check_num==1:
			continue

		#print(len(dmsa[s][c-12:c]),len(dmsa[s][c+1:c+13]))
		#exit()

		lseq=''
		rseq=''
		for l in left:
			if len(lseq)==(k-1)/2:
				break
			if l in dmap:
				lseq+=dmap[l]
			elif not l=='-':
				lseq+=l.upper()
		for r in right:
			if len(rseq)==(k-1)/2:
				break
			if r in dmap:
				rseq+=dmap[r]
			elif not r=='-':
				rseq+=r.upper()
		lseq=lseq[::-1]
		kmr=lseq+base+rseq
		if len(kmr) <k:
			if len(lseq)<(k-1)/2:
				rseq=''
				rl=k-1-len(lseq)
				for r in right:
					if len(rseq)==rl:
						break
					if r in dmap:
						rseq+=dmap[r]
					elif not r=='-':
						rseq+=r.upper()
			if len(rseq)<(k-1)/2:
				lseq=''
				ll=k-1-len(rseq)
				for l in left:
					if len(lseq)==ll:
						break
					if l in dmap:
						lseq+=dmap[l]
					elif not l=='-':
						lseq+=l.upper()
			kmr=lseq+base+rseq
		if c not in dpbk:
			dpbk[c]={base:{kmr:1}}
		else:
			if base not in dpbk[c]:
				dpbk[c][base]={kmr:1}
			else:
				dpbk[c][base][kmr]=1
		if kmr not in dkl:
			dkl[kmr]={ds2id[s]:''}
		else:
			dkl[kmr][ds2id[s]]=''
		if kmr not in dkpb:
			dkpb[kmr]={pos_base:''}
		else:
			dkpb[kmr][pos_base]=''

o=open(out,'w+')
o3=open(out2,'w+')
i=1
for k in dkpb:
	info=','.join(list(dkpb[k].keys()))
	info2=','.join(list(dkl[k].keys()))
	o.write(k+'\t'+info+'\t'+info2+'\n')
	o3.write('>'+str(i)+'\n'+k+'\n')
	seq=Seq(k,IUPAC.ambiguous_dna)
	i+=1
	k2=seq.reverse_complement()._data
	o.write(k2+'\t'+info+'\t'+info2+'\n')
	o3.write('>'+str(i)+'\n'+k2+'\n')
	i+=1
print('Kmer POS-Snp file done! Start generating matrix.')
#### Generate Strain pos-snp matrix ####

o2=open('Strain_pos_snp_matrix_consider_all.txt','w+')
base_arr=['A','T','G','C']
pos_snp=[]
for c in column_arr:
	for b in base_arr:
		pb=str(c)+'-'+b
		pos_snp.append(pb)
fl='\t'.join(pos_snp)
o2.write('\t\t'+fl+'\n')
for s in dmsa:
	temd={}
	for c in column_arr:
		if dmsa[s][c] not in dmap:
			temd[str(c)+'-N']=''
		else:
			temd[str(c)+'-'+dmap[dmsa[s][c]]]=''
	o2.write(s)
	for p in pos_snp:
		if p not in temd:
			o2.write('\t0')	
		else:
			o2.write('\t1')
	o2.write('\n')

